module Htpasswd
  def self.included(base)
    base.extend(ClassMethods)
  end
  
  class Error < StandardError
    define_method(:message) { 'Authorization is required' }
  end

  module ClassMethods
    def htpasswd(options={})
      if options[:file]
        type = options[:type] || :crypted
        IO.foreach(options[:file]) do |line|
          user, pass = line.chomp.split(':', 2)
          write_inheritable_array(:htpasswd, [{:user => user, :pass => pass, :type => type }])
        end
      else
        write_inheritable_array(:htpasswd, [{:user => options[:user], :pass => options[:pass], :type => :plain }])
      end
      
      skip_before_filter :htpasswd_authorize rescue nil
      before_filter :htpasswd_authorize
    end
  end
  
  protected
  def htpasswd_authorize
    raise Htpasswd::Error unless auth_header = request.env['HTTP_AUTHORIZATION']
    auth_type, auth_data = auth_header.split(' ', 2)
    user, pass = Base64.decode64(auth_data).split(':', 2)
    raise Htpasswd::Error unless pass.length > 0
    auth_users = self.class.read_inheritable_attribute(:htpasswd)
    #logger.info [user, pass].inspect
    #logger.info auth_users.inspect

    auth_success = false
    auth_users.each do |h|
      case h[:type]
      when :plain
        if h[:user] == user and h[:pass] == pass
          auth_success = true
          break
        end
      when :crypted
        if h[:user] == user and h[:pass] == pass.crypt(h[:pass])
          auth_success = true
          break
        end
      end
    end
    raise Htpasswd::Error unless auth_success
    return true
  rescue Htpasswd::Error => error
    realm = "Authorization"
    response.headers['WWW-Authenticate'] = response.headers['WWW-Authenticate'].to_a
    response.headers['WWW-Authenticate'].push("Basic realm=\"#{realm}\"")
    #headers['Status'] = '401 Unauthorized'
    render :nothing => true, :status => 401
    return false
  end
end